<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\MagentoToRefiloProductAdapter;
use FiloBlu\Refilo\Remote\Entity\MagentoToRefiloProductAdapterFactory;
use FiloBlu\Refilo\Remote\Entity\Product;
use LogicException;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;

/**
 * Provider implements also a SPL iterator interface
 *  in foreach() the calling order<br>
 *  -> rewind<br>
 *  -> valid<br>
 *  -> current<br>
 *  -> key<br>
 * Class ProductProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class ProductProvider extends BaseProvider
{
    /**
     * @var int
     */
    protected $pageSize = 500;

    /**
     * @var SearchCriteriaInterface
     */
    protected $searchCriteria;

    /**
     * @var StoreInterface
     */
    protected $store;

    /**
     * @var false|float
     */
    protected $totalPages;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var MagentoToRefiloProductAdapterFactory
     */
    private $productAdapterFactory;

    /**
     * @var MagentoToRefiloProductAdapter;
     */
    private $productAdapter;

    /**
     * @var ProductRepository
     */
    private $productRepository;

    /**
     * @var Status
     */
    private $status;

    /**
     * @var SearchCriteriaBuilderFactory
     */
    private $searchCriteriaBuilderFactory;

    /**
     * ProductProvider constructor.
     * @param MagentoToRefiloProductAdapterFactory $productAdapterFactory
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     * @param ProductRepository $productRepository
     * @param Status $status
     */
    public function __construct(
        MagentoToRefiloProductAdapterFactory $productAdapterFactory,
        SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
        ProductRepository $productRepository,
        Status $status,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        $this->productAdapterFactory = $productAdapterFactory;
        $this->productRepository = $productRepository;
        $this->status = $status;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->store = $store;
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        throw new LogicException('Count not filter by website');
    }

    /**
     * @return bool
     */
    public function valid()
    {
        if ($this->searchCriteria->getCurrentPage() < $this->totalPages) {

            if (current($this->items) !== false) {
                return true;
            }

            $this->handleItems($this->productRepository->getList(
                $this->searchCriteria->setCurrentPage($this->searchCriteria->getCurrentPage() + 1)
            ));

            return true;
        }

        return current($this->items) !== false;
    }

    /**
     * @param $items
     */
    public function handleItems(\Magento\Catalog\Api\Data\ProductSearchResultsInterface $result)
    {
        $this->items = [];

        /** @var ProductInterface $product */
        foreach ($result->getItems() as $product) {
            $this->items[(int)$product->getId()] = $this->productAdapter->adapt($product, new Product());
        }

        if (!$this->hasReadHandler()) {
            return;
        }

        $this->getReadHandler()->onRead($this->items, $this->pageSize, $this->getReadHandlerArguments());
    }

    /**
     *
     */
    public function rewind()
    {
        $this->items = [];

        $this->productAdapter = $this->productAdapterFactory->create();

        $pageSize = (int)$this->scopeConfig->getValue('sync/data/index_bulk_size');
        $this->pageSize = ($pageSize) ? $pageSize : $this->pageSize;

        $builder = $this->searchCriteriaBuilderFactory->create();

        if (!empty($this->ids)) {
            $builder->addFilter('entity_id', $this->ids, 'in');
        }

        if ($this->store !== null) {
            $builder->addFilter('store_id', $this->store->getId(), 'eq');
        }

        $this->searchCriteria = $builder->create();
        $this->searchCriteria->setPageSize($this->pageSize)->setCurrentPage(1);
        $result = $this->productRepository->getList($this->searchCriteria);

        $this->totalPages = ceil($result->getTotalCount() / $this->pageSize);

        $this->handleItems($result);
    }
}
